@@ -51,6 +51,7 @@ removeTable  = core.DB.removeTable
 removeTables  = core.DB.removeTables
 
 find         = core.DB.getInventory
+findp        = core.DB.findp
 findExact    = core.DB.findExact
 getTable     = core.DB.getTable
 getTables    = core.DB.getTables
@@ -82,7 +83,10 @@ validate_ID = core.DB.validate_ID
 import_new_source_from_remote = core.DB.importSourceFromRemote
 export_new_source_to_remote   = core.DB.exportSourceToRemote
 remove_source                 = core.DB.removeSource
+push_source_to_remote         = core.DB.gitManager.push_to_remote_datashelf
+pull_source_from_remote      = core.DB.gitManager.pull_update_from_remote
 
+import datatoolbox.tools as tools
 
 if config.PATH_TO_DATASHELF == os.path.join(config.MODULE_PATH, 'data/SANDBOX_datashelf'):
     print("""
