@@ -6,12 +6,14 @@ Created on Wed May  8 11:25:15 2019
 @author: Andreas Geiges
 """
 import pandas as pd
-import numpy  as np
+import numpy as np
+from typing import Union, Iterable, Optional
+
 import datatoolbox as dt
 from datatoolbox import mapping as mapp
 from datatoolbox import core
 from datatoolbox import config
-#from .data_structures import read_csv
+from datatoolbox import data_structures
 from datatoolbox.greenhouse_gas_database import GreenhouseGasTable 
 import matplotlib.pylab as plt
 import os
@@ -252,12 +254,16 @@ def scatterIndicatorComparison(tableX, tableY):
     
 def cleanDataTable(dataTable):
     # ensure valid ISO or region IDs
-    dataTable = dataTable.filter(spaceIDs = mapp.getValidSpatialIDs())    
-    
-    dataTable = dataTable.loc[:,~dataTable.isnull().all(axis=0)]
-    dataTable = dataTable.loc[~dataTable.isnull().all(axis=1),:]
-    #dataTable = dataTable.loc[~dataTable.isnull().all(axis=0),:]
+    invalidSpatialIDs = dataTable.index.difference(mapp.getValidSpatialIDs())
+    if not invalidSpatialIDs.empty:
+        print(
+            f"Removing unknown regions from dataTable {dataTable.meta.get('ID', 'unnamed')}:",
+            ", ".join(invalidSpatialIDs)
+        )
+        dataTable = dataTable.loc[dataTable.index.difference(invalidSpatialIDs)]
     
+    dataTable = dataTable.dropna(how="all", axis=1).dropna(how="all", axis=0)
+
     # clean meta data
     keysToDelete = list()
     for key in dataTable.meta.keys():
@@ -953,7 +959,125 @@ def plotTables(tableList, countryList):
             plt.plot(table.columns, table.loc[coISO,:].T, color =coList[i] )
     plt.legend(countryList)
     plt.title(table.ID)
+
+
+def pattern_match(
+    data: pd.Series,
+    patterns: Union[str, Iterable[str]],
+    regex : bool = False
+):
+    """Find matches in `data` for a list of shell-style `patterns`
+
+    Arguments
+    ---------
+    data : pd.Series
+        Series of data to match against
+    patterns : Union[str, Iterable[str]]
+        One or multiple patterns, which are OR'd together
+    regex : bool, optional
+        Accept plain regex syntax instead of shell-style, default: False
     
+    Returns
+    -------
+    matches : pd.Series
+        Mask for selecting matched rows
+    """
+
+    if not isinstance(patterns, Iterable) or isinstance(patterns, str):
+        patterns = [patterns]
+    elif not patterns:
+        raise ValueError("pattern list may not be empty")
+
+    matches = False
+    for pat in patterns:
+        if isinstance(pat, str):
+            if not regex:
+                pat = shell_pattern_to_regex(pat) + '$'
+            matches |= data.str.match(pat, na=False)
+        else:
+            matches |= data == pat
+
+    return matches
+
+
+def shell_pattern_to_regex(s):
+    """Escape characters with specific regexp use"""
+    return (
+        str(s)
+        .replace('|', r'\|')
+        .replace('.', r'\.')  # `.` has to be replaced before `*`
+        .replace('**', '__starstar__') # temporarily __starstar__
+        .replace('*', r'[^|]*')
+        .replace('__starstar__', r'.*')
+        .replace('+', r'\+')
+        .replace('(', r'\(')
+        .replace(')', r'\)')
+        .replace('$', r'\$')
+    )
+
+def fix_source_inconsistency(sourceID):
+    gitManager = dt.core.DB.gitManager.__dict__['repositories'][sourceID]
+    gitManager.git.execute(["git", "add", "tables/*"])
+    dt.core.DB.gitManager.sources.loc['IEA_WEB_2020','git_commit_hash'] = gitManager.commit().hexsha
+    dt.core.DB.gitManager.commit('inconsistent fix')
+
+
+def _create_sandbox_tables(sourceID, random_seed):
+    #%%
+#    import datatoolbox as dt
+    import numpy as np
+    np.random.seed(1)
+    
+
+    tables = list()
+    source_meta = {'SOURCE_ID': sourceID,
+                      'collected_by' : 'Hard worker 1',
+                      'date': core.getDateString(),
+                      'source_url' : 'www.www.www',
+                      'licence': 'open access' }
+    
+    meta = {'entity' : 'Emissions|CO2',
+            'category': 'Total',
+            'scenario': None,
+            'model' : 'Historic',
+            'source' : sourceID,
+            'unit' : 'Mt CO2'}
+    
+    table = data_structures.Datatable(np.random.randint(0,20,[3,21]), 
+                         columns = list(range(2000,2021)), 
+                         index = ['World', 'Asia', 'ZAF'],
+                         meta=meta).astype(float)
+    tables.append(table)
+    
+    meta = {'entity' : 'Emissions|CO2',
+            'category': 'Total',
+            'scenario': 'Medium',
+            'model' : 'Projection',
+            'source' : sourceID,
+            'unit' : 'Mt CO2'}
+    
+    table = data_structures.Datatable(np.random.randint(20,30,[3,31]), 
+                         columns = list(range(2020,2051)), 
+                         index = ['World', 'Asia', 'ZAF'],
+                         meta=meta).astype(float)
+    tables.append(table)
+    
+    
+    
+    meta = {'entity' : 'Emissions|CO2',
+            'category': 'Total_excl_LULUCF',
+            'scenario': None,
+            'model' : 'Historic',
+            'source_name' : sourceID,
+            'unit' : 'Mt CO2'}
+    
+    table = data_structures.Datatable(np.random.randint(0,15,[3,21]), 
+                         columns = list(range(2000,2021)), 
+                         index = ['World', 'Asia', 'ZAF'],
+                         meta=meta).astype(float)
+    tables.append(table)
+    return tables,source_meta
+
 #%%    
 if __name__ == '__main__':
     #%%
